@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2004 Imendio AB
+ * Copyright (C) 2004-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,20 +16,24 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Mikael Hallendal <micke@imendio.com>
+ *          Martyn Russell <martyn@imendio.com>
  */
 
-#include <config.h>
+#include "config.h"
 
-#include <glib/gi18n.h>
 #include <string.h>
 
-#include "gossip-utils.h"
+#include <glib/gi18n.h>
+
 #include "gossip-contact.h"
+#include "gossip-utils.h"
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CONTACT, GossipContactPriv))
 
 /* Make sure any additions to the private contact structure are copied
- * in the gossip_contact_copy() function further down.
+ * in the gossip_contact_copy_values() function further down.
  */
 typedef struct _GossipContactPriv GossipContactPriv;
 
@@ -43,34 +47,35 @@ struct _GossipContactPriv {
 	GList              *groups;
 
 	GossipSubscription  subscription;
-
-	guchar             *avatar;
-	guint               avatar_size;
-
+	GossipAvatar       *avatar;
 	GossipAccount      *account;
 };
 
-static void contact_class_init   (GossipContactClass *class);
-static void contact_init         (GossipContact      *contact);
-static void contact_finalize     (GObject            *object);
-static void contact_get_property (GObject            *object,
-				  guint               param_id,
-				  GValue             *value,
-				  GParamSpec         *pspec);
-static void contact_set_property (GObject            *object,
-				  guint               param_id,
-				  const GValue       *value,
-				  GParamSpec         *pspec);
+static void contact_class_init    (GossipContactClass *class);
+static void contact_init          (GossipContact      *contact);
+static void contact_finalize      (GObject            *object);
+static void contact_get_property  (GObject            *object,
+				   guint               param_id,
+				   GValue             *value,
+				   GParamSpec         *pspec);
+static void contact_set_property  (GObject            *object,
+				   guint               param_id,
+				   const GValue       *value,
+				   GParamSpec         *pspec);
+static void contact_set_type      (GossipContact      *contact,
+				   GossipContactType   type);
+static void contact_set_presences (GossipContact      *contact,
+				   GList              *presences);
 
 enum {
 	PROP_0,
 	PROP_TYPE,
 	PROP_NAME,
 	PROP_ID,
-	PROP_ACTIVE_PRESENCE,
+	PROP_PRESENCES,
+	PROP_GROUPS,
 	PROP_SUBSCRIPTION,
 	PROP_AVATAR,
-	PROP_AVATAR_SIZE,
 	PROP_ACCOUNT
 };
 
@@ -141,12 +146,18 @@ contact_class_init (GossipContactClass *class)
 							      G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class,
-					 PROP_ACTIVE_PRESENCE,
-					 g_param_spec_object ("active-presence",
-							      "Contact active presence",
-							      "Current presence of contact",
-							      GOSSIP_TYPE_PRESENCE,
-							      G_PARAM_READABLE));
+					 PROP_PRESENCES,
+					 g_param_spec_pointer ("presences",
+							       "Contact presences",
+							       "Presences of contact",
+							       G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class,
+					 PROP_GROUPS,
+					 g_param_spec_pointer ("groups",
+							       "Contact groups",
+							       "Groups of contact",
+							       G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class,
 					 PROP_SUBSCRIPTION,
@@ -160,19 +171,11 @@ contact_class_init (GossipContactClass *class)
 
 	g_object_class_install_property (object_class,
 					 PROP_AVATAR,
-					 g_param_spec_pointer ("avatar",
-							       "Avatar image",
-							       "The avatar image",
-							       G_PARAM_READWRITE));
-	g_object_class_install_property (object_class,
-					 PROP_AVATAR_SIZE,
-					 g_param_spec_uint ("avatar_size",
-							    "Avatar size",
-							    "The size of the avatar image",
-							    0,
-							    G_MAXUINT,
-							    0,
-							    G_PARAM_READWRITE));
+					 g_param_spec_boxed ("avatar",
+							     "Avatar image",
+							     "The avatar image",
+							     GOSSIP_TYPE_AVATAR,
+							     G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class,
 					 PROP_ACCOUNT,
@@ -199,7 +202,6 @@ contact_init (GossipContact *contact)
 	priv->account     = NULL;
 	priv->groups      = NULL;
 	priv->avatar      = NULL;
-	priv->avatar_size = 0;
 }
 
 static void
@@ -211,7 +213,10 @@ contact_finalize (GObject *object)
 
 	g_free (priv->name);
 	g_free (priv->id);
-	g_free (priv->avatar);
+
+	if (priv->avatar) {
+		gossip_avatar_unref (priv->avatar);
+	}
 
 	if (priv->presences) {
 		g_list_foreach (priv->presences, (GFunc) g_object_unref, NULL);
@@ -223,7 +228,9 @@ contact_finalize (GObject *object)
 		g_list_free (priv->groups);
 	}
 
-	g_object_unref (priv->account);
+	if (priv->account) {
+		g_object_unref (priv->account);
+	}
 
 	(G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
@@ -250,28 +257,20 @@ contact_get_property (GObject    *object,
 		g_value_set_string (value,
 				    gossip_contact_get_id (GOSSIP_CONTACT (object)));
 		break;
-	case PROP_ACTIVE_PRESENCE:
-		if (priv->presences) {
-			g_value_set_object (value, G_OBJECT (priv->presences->data));
-		} else {
-			g_value_set_object (value, NULL);
-		}
+	case PROP_PRESENCES:
+		g_value_set_pointer (value, priv->presences);
+		break;
+	case PROP_GROUPS:
+		g_value_set_pointer (value, priv->groups);
 		break;
 	case PROP_SUBSCRIPTION:
 		g_value_set_int (value, priv->subscription);
 		break;
 	case PROP_AVATAR:
-		g_value_set_pointer (value, priv->avatar);
-		break;
-	case PROP_AVATAR_SIZE:
-		g_value_set_uint (value, priv->avatar_size);
+		g_value_set_boxed (value, priv->avatar);
 		break;
 	case PROP_ACCOUNT:
-		if (priv->account) {
-			g_value_set_object (value, G_OBJECT (priv->account));
-		} else {
-			g_value_set_object (value, NULL);
-		}
+		g_value_set_object (value, priv->account);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
@@ -291,7 +290,8 @@ contact_set_property (GObject      *object,
 
 	switch (param_id) {
 	case PROP_TYPE:
-		priv->type = g_value_get_int (value);
+		contact_set_type (GOSSIP_CONTACT (object),
+				  g_value_get_int (value));
 		break;
 	case PROP_NAME:
 		gossip_contact_set_name (GOSSIP_CONTACT (object),
@@ -301,14 +301,21 @@ contact_set_property (GObject      *object,
 		gossip_contact_set_id (GOSSIP_CONTACT (object),
 				       g_value_get_string (value));
 		break;
+	case PROP_PRESENCES:
+		contact_set_presences (GOSSIP_CONTACT (object),
+				       g_value_get_pointer (value));
+		break;
+	case PROP_GROUPS:
+		gossip_contact_set_groups (GOSSIP_CONTACT (object),
+					   g_value_get_pointer (value));
+		break;
 	case PROP_SUBSCRIPTION:
-		priv->subscription = g_value_get_int (value);
+		gossip_contact_set_subscription (GOSSIP_CONTACT (object),
+						 g_value_get_int (value));
 		break;
 	case PROP_AVATAR:
-		priv->avatar = g_value_get_pointer (value);
-		break;
-	case PROP_AVATAR_SIZE:
-		priv->avatar_size = g_value_get_uint (value);
+		gossip_contact_set_avatar (GOSSIP_CONTACT (object),
+					   g_value_get_boxed (value));
 		break;
 	case PROP_ACCOUNT:
 		gossip_contact_set_account (GOSSIP_CONTACT (object),
@@ -347,41 +354,34 @@ gossip_contact_new_full (GossipContactType  type,
 GossipContact *
 gossip_contact_copy (GossipContact *contact)
 {
-	GossipContact     *new_contact;
-	GossipContactPriv *new_priv;
-	GossipContactPriv *priv;
-	GList             *l;
+	GossipContact *new_contact;
 
-	g_return_val_if_fail (GOSSIP_IS_CONTACT (contact), NULL);
-
-	priv = GET_PRIV (contact);
-
-	new_contact = gossip_contact_new_full (priv->type,
-					       priv->account,
-					       priv->id,
-					       priv->name);
-
-	new_priv = GET_PRIV (new_contact);
-
-	new_priv->subscription = priv->subscription;
-
-	for (l = priv->groups; l; l = l->next) {
-		new_priv->groups = g_list_prepend (new_priv->groups,
-						   g_strdup (l->data));
-	}
-
-	priv->groups = g_list_reverse (priv->groups);
-
-	for (l = priv->presences; l; l = l->next) {
-		new_priv->presences = g_list_prepend (new_priv->presences,
-						      g_strdup (l->data));
-	}
-	
-	priv->presences = g_list_reverse (priv->presences);
+	new_contact = g_object_new (GOSSIP_TYPE_CONTACT, NULL);
+	gossip_contact_copy_values (contact, new_contact);
 
 	return new_contact;
 }
 
+void
+gossip_contact_copy_values (GossipContact *contact_from,
+			    GossipContact *contact_to)
+{
+	GossipContactPriv *from_priv;
+
+	g_return_if_fail (GOSSIP_IS_CONTACT (contact_from));
+	g_return_if_fail (GOSSIP_IS_CONTACT (contact_to));
+
+	from_priv = GET_PRIV (contact_from);
+
+	gossip_contact_set_name (contact_to, from_priv->name);
+	gossip_contact_set_id (contact_to, from_priv->id);
+	gossip_contact_set_subscription (contact_to, from_priv->subscription);
+	gossip_contact_set_avatar (contact_to, from_priv->avatar);
+	gossip_contact_set_account (contact_to, from_priv->account);
+	gossip_contact_set_groups (contact_to, from_priv->groups);
+	contact_set_presences (contact_to, from_priv->presences);
+}
+
 GossipContactType
 gossip_contact_get_type (GossipContact *contact)
 {
@@ -427,22 +427,16 @@ gossip_contact_get_name (GossipContact *contact)
 	return priv->name;
 }
 
-const guchar *
-gossip_contact_get_avatar (GossipContact *contact,
-			   gsize         *avatar_size)
+GossipAvatar *
+gossip_contact_get_avatar (GossipContact *contact)
 {
 	GossipContactPriv *priv;
-	guchar            *avatar;
 
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (contact), NULL);
-	g_return_val_if_fail (avatar_size != NULL, NULL);
 
 	priv = GET_PRIV (contact);
 
-	avatar = (guchar*) priv->avatar;
-	*avatar_size = priv->avatar_size;
-
-	return avatar;
+	return priv->avatar;
 }
 
 GossipAccount *
@@ -535,6 +529,19 @@ gossip_contact_get_subscription (GossipContact *contact)
 	return priv->subscription;
 }
 
+static void
+contact_set_type (GossipContact      *contact,
+		  GossipContactType   type)
+{
+	GossipContactPriv *priv;
+
+	priv = GET_PRIV (contact);
+
+	priv->type = type;
+
+	g_object_notify (G_OBJECT (contact), "type");
+}
+
 void
 gossip_contact_set_id (GossipContact *contact,
 		       const gchar   *id)
@@ -569,10 +576,28 @@ gossip_contact_set_name (GossipContact *contact,
 	g_object_notify (G_OBJECT (contact), "name");
 }
 
+static void
+contact_set_presences (GossipContact *contact,
+		       GList         *presences)
+{
+	GossipContactPriv *priv;
+
+	priv = GET_PRIV (contact);
+
+	if (priv->presences) {
+		g_list_foreach (priv->presences, (GFunc) g_object_unref, NULL);
+		g_list_free (priv->presences);
+	}
+
+	priv->presences = g_list_copy (presences);
+	g_list_foreach (priv->presences, (GFunc) g_object_ref, NULL);
+
+	g_object_notify (G_OBJECT (contact), "presences");
+}
+
 void
 gossip_contact_set_avatar (GossipContact *contact,
-			   const guchar  *avatar,
-			   gsize          avatar_size)
+			   GossipAvatar  *avatar)
 {
 	GossipContactPriv *priv;
 
@@ -580,18 +605,16 @@ gossip_contact_set_avatar (GossipContact *contact,
 
 	priv = GET_PRIV (contact);
 
-	g_free (priv->avatar);
+	if (priv->avatar) {
+		gossip_avatar_unref (priv->avatar);
+		priv->avatar = NULL;
+	}
 
 	if (avatar) {
-		priv->avatar = g_memdup (avatar, avatar_size);
-		priv->avatar_size = avatar_size;
-	} else {
-		priv->avatar = NULL;
-		priv->avatar_size = 0;
+		priv->avatar = gossip_avatar_ref (avatar);
 	}
 
 	g_object_notify (G_OBJECT (contact), "avatar");
-	g_object_notify (G_OBJECT (contact), "avatar-size");
 }
 
 void
@@ -623,34 +646,47 @@ gossip_contact_add_presence (GossipContact  *contact,
 			     GossipPresence *presence)
 {
 	GossipContactPriv *priv;
+	GossipPresence    *this_presence;
 	GList             *l;
-	gboolean           old;
 
 	g_return_if_fail (GOSSIP_IS_CONTACT (contact));
 	g_return_if_fail (GOSSIP_IS_PRESENCE (presence));
 
 	priv = GET_PRIV (contact);
 
-	old = FALSE;
+	for (l = priv->presences; l && presence; l = l->next) {
+		this_presence = l->data;
 
-	for (l = priv->presences; l; l = l->next) {
-		if (gossip_presence_resource_equal (l->data, presence)) {
-			/* Replace the old presence with the new */
-			g_object_unref (l->data);
-			l->data = g_object_ref (presence);
-			old = TRUE;
+		if (gossip_presence_resource_equal (this_presence, presence)) {
+			gint ref_count;
+
+			ref_count = G_OBJECT (presence)->ref_count;
+
+			/* Remove old presence for this resource, we
+			 * would use g_list_remove_all() here but we
+			 * want to make sure we unref for each
+			 * instance we find it in the list.
+			 */
+			priv->presences = g_list_remove (priv->presences, this_presence);
+			g_object_unref (this_presence);
+
+			if (!priv->presences || ref_count <= 1) {
+				break;
+			}
+
+			/* Reset list to beginnging to make sure we
+			 * didn't miss any duplicates.
+			 */
+			l = priv->presences;
 		}
 	}
 
-	if (!old) {
-		priv->presences = g_list_prepend (priv->presences,
-						  g_object_ref (presence));
-	}
+	/* Add new presence */
+	priv->presences = g_list_insert_sorted (priv->presences,
+						g_object_ref (presence),
+						gossip_presence_sort_func);
 
-	priv->presences = g_list_sort (priv->presences,
-				       gossip_presence_priority_sort_func);
-	/* Highest priority first */
-	priv->presences = g_list_reverse (priv->presences);
+	g_object_notify (G_OBJECT (contact), "presences");
 }
 
 void
@@ -658,6 +694,7 @@ gossip_contact_remove_presence (GossipContact  *contact,
 				GossipPresence *presence)
 {
 	GossipContactPriv *priv;
+	GossipPresence    *this_presence;
 	GList             *l;
 
 	g_return_if_fail (GOSSIP_IS_CONTACT (contact));
@@ -666,32 +703,46 @@ gossip_contact_remove_presence (GossipContact  *contact,
 	priv = GET_PRIV (contact);
 
 	for (l = priv->presences; l; l = l->next) {
-		if (gossip_presence_resource_equal (l->data, presence)) {
-			break;
+		this_presence = l->data;
+
+		if (gossip_presence_resource_equal (this_presence, presence)) {
+			gint ref_count;
+
+			ref_count = G_OBJECT (presence)->ref_count;
+
+			/* Remove old presence for this resource, we
+			 * would use g_list_remove_all() here but we
+			 * want to make sure we unref for each
+			 * instance we find it in the list.
+			 */
+			priv->presences = g_list_remove (priv->presences, this_presence);
+			g_object_unref (this_presence);
+
+			if (!priv->presences || ref_count <= 1) {
+				break;
+			}
+
+			/* Reset list to beginnging to make sure we
+			 * didn't miss any duplicates.
+			 */
+			l = priv->presences;
 		}
 	}
 
-	if (l) {
-		priv->presences = g_list_remove_link (priv->presences, l);
-
-		g_object_unref (l->data);
-		g_list_free_1 (l);
-	}
-
 	priv->presences = g_list_sort (priv->presences,
-				       gossip_presence_priority_sort_func);
-	/* Highest priority first */
-	priv->presences = g_list_reverse (priv->presences);
+				       gossip_presence_sort_func);
+
+	g_object_notify (G_OBJECT (contact), "presences");
 }
 
-gboolean
+void
 gossip_contact_set_groups (GossipContact *contact,
 			   GList         *groups)
 {
 	GossipContactPriv *priv;
 	GList             *old_groups, *l;
 
-	g_return_val_if_fail (GOSSIP_IS_CONTACT (contact), FALSE);
+	g_return_if_fail (GOSSIP_IS_CONTACT (contact));
 
 	priv = GET_PRIV (contact);
 
@@ -706,7 +757,7 @@ gossip_contact_set_groups (GossipContact *contact,
 	g_list_foreach (old_groups, (GFunc) g_free, NULL);
 	g_list_free (old_groups);
 
-	return TRUE;
+	g_object_notify (G_OBJECT (contact), "groups");
 }
 
 void
@@ -724,23 +775,6 @@ gossip_contact_set_subscription (GossipContact      *contact,
 	g_object_notify (G_OBJECT (contact), "subscription");
 }
 
-gint
-gossip_contact_compare (gconstpointer a,
-			gconstpointer b)
-{
-	const gchar *id_a;
-	const gchar *id_b;
-
-	g_return_val_if_fail (GOSSIP_IS_CONTACT (a), 0);
-	g_return_val_if_fail (GOSSIP_IS_CONTACT (b), 0);
-
-	id_a = gossip_contact_get_id (GOSSIP_CONTACT (a));
-	id_b = gossip_contact_get_id (GOSSIP_CONTACT (b));
-
-	/* FIXME: Maybe use utf8 strcmp? At least jabber ids are UTF8. */
-	return strcmp (id_a, id_b);
-}
-
 gint
 gossip_contact_name_compare (gconstpointer a,
 			     gconstpointer b)
@@ -782,27 +816,38 @@ gboolean
 gossip_contact_equal (gconstpointer v1,
 		      gconstpointer v2)
 {
-	const gchar *id_a;
-	const gchar *id_b;
+	GossipAccount *account_a;
+	GossipAccount *account_b;
+	const gchar   *id_a;
+	const gchar   *id_b;
 
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (v1), FALSE);
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (v2), FALSE);
 
+	account_a = gossip_contact_get_account (GOSSIP_CONTACT (v1));
+	account_b = gossip_contact_get_account (GOSSIP_CONTACT (v2));
+
 	id_a = gossip_contact_get_id (GOSSIP_CONTACT (v1));
 	id_b = gossip_contact_get_id (GOSSIP_CONTACT (v2));
 
-	return g_str_equal (id_a, id_b);
+	return gossip_account_equal (account_a, account_b) && g_str_equal (id_a, id_b);
 }
 
 guint
 gossip_contact_hash (gconstpointer key)
 {
+	GossipContactPriv *priv;
+	guint              hash = 0;
+
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (key), +1);
 
-	return g_str_hash (gossip_contact_get_id (GOSSIP_CONTACT (key)));
-}
+	priv = GET_PRIV (GOSSIP_CONTACT (key));
 
-/* convenience functions */
+	hash += gossip_account_hash (gossip_contact_get_account (GOSSIP_CONTACT (key)));
+	hash += g_str_hash (gossip_contact_get_id (GOSSIP_CONTACT (key)));
+
+	return hash;
+}
 
 gboolean
 gossip_contact_is_online (GossipContact *contact)
