@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003 Martyn Russell <ginxd@btopenworld.com>
+ * Copyright (C) 2003-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,17 +16,26 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Martyn Russell <martyn@imendio.com>
  */
 
-#include <config.h>
-#include <string.h>
-#include <gtk/gtk.h>
-#include <glib/gi18n.h>
+#include "config.h"
 
+#include <string.h>
+
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
+#include <glade/glade.h>
+
+#include <libgossip/gossip-account.h>
+#include <libgossip/gossip-session.h>
 #include <libgossip/gossip-utils.h>
+#include <libgossip/gossip-vcard.h>
 
 #include "gossip-app.h"
 #include "gossip-edit-contact-dialog.h"
+#include "gossip-ui-utils.h"
 
 typedef struct {
 	GtkWidget       *dialog;
@@ -44,8 +53,6 @@ typedef struct {
 	GossipContact   *contact;
 	GtkCellRenderer *renderer;
 	gboolean         changes_made;
-
-	gulong           contact_signal_handler;
 } GossipEditContactDialog;
 
 typedef struct {
@@ -90,8 +97,7 @@ static gboolean edit_contact_dialog_model_find_selected_foreach  (GtkTreeModel
 								  GtkTreeIter             *iter,
 								  FindSelected            *data);
 static void     edit_contact_dialog_update_widgets               (GossipContact           *contact);
-static void     edit_contact_dialog_contact_updated_cb           (GossipSession           *session,
-								  GossipContact           *contact,
+static void     edit_contact_dialog_contact_updated_cb           (GossipContact           *contact,
 								  gpointer                 user_data);
 static void     edit_contact_dialog_cell_toggled                 (GtkCellRendererToggle   *cell,
 								  gchar                   *path_string,
@@ -435,8 +441,7 @@ edit_contact_dialog_update_widgets (GossipContact *contact)
 }
 
 static void
-edit_contact_dialog_contact_updated_cb (GossipSession           *session,
-					GossipContact           *contact,
+edit_contact_dialog_contact_updated_cb (GossipContact           *contact,
 					gpointer                 user_data)
 {
 	edit_contact_dialog_update_widgets (contact);	
@@ -641,13 +646,11 @@ edit_contact_dialog_destroy_cb (GtkWidget               *widget,
 		g_object_unref (dialog->renderer);
 	}
 
+	g_signal_handlers_disconnect_by_func (dialog->contact,
+					      edit_contact_dialog_contact_updated_cb,
+					      NULL);
+
 	g_hash_table_remove (dialogs, dialog->contact);
-
-	if (dialog->contact_signal_handler) {
-		g_signal_handler_disconnect (gossip_app_get_session (),
-					     dialog->contact_signal_handler);
-	}
-
 	g_free (dialog);
 }
 
@@ -656,13 +659,11 @@ gossip_edit_contact_dialog_show (GossipContact *contact,
 				 GtkWindow     *parent)
 {
 	GossipEditContactDialog *dialog;
-	GossipSession           *session;
 	GladeXML                *glade;
 	GList                   *groups;
 	GtkSizeGroup            *size_group;
 	gchar                   *id_escaped;
 	gchar                   *str;
-	guint                    id;
 
 	g_return_if_fail (GOSSIP_IS_CONTACT (contact));
 
@@ -727,12 +728,9 @@ gossip_edit_contact_dialog_show (GossipContact *contact,
 	g_free (str);
 
 	/* Subscription listener */
-	session = gossip_app_get_session ();
-	id = g_signal_connect (session,
-			       "contact-updated",
-			       G_CALLBACK (edit_contact_dialog_contact_updated_cb),
-			       NULL);
-	dialog->contact_signal_handler = id;
+	g_signal_connect (contact, "notify",
+			  G_CALLBACK (edit_contact_dialog_contact_updated_cb),
+			  NULL);
 
 	/* Set up contact subscription widgets */
 	edit_contact_dialog_update_widgets (contact);	
